#include <PID_marco.h>


#include <NewPing.h>

/*
Control of a Rover5 robot

Copyright (c) 2013, Marco Forgione
All rights reserved.
http://www.dcsc.tudelft.nl/~mforgione/extra.html
*/

#include <stdio.h>
#include <string.h>


#define LEFT_MOTOR_DIRECTION 8
#define LEFT_MOTOR_PWM 9
#define LEFT_ENCODER_INTERRUPT 0 // PIN 2!
#define LEFT_ENCODER_CHA 4
#define LEFT_ENCODER_CHB 5

#define RIGHT_MOTOR_DIRECTION 13
#define RIGHT_MOTOR_PWM 11
#define RIGHT_ENCODER_INTERRUPT 1 // PIN 3!
#define RIGHT_ENCODER_CHA 6
#define RIGHT_ENCODER_CHB 7

#define TRIGGER_PIN 10
#define ECHO_PIN 12

char buf[50];
const int QEM [16] = {0,-1,1,2,1,0,2,-1,-1,2,0,1,2,1,-1,0}; // Quadrature Encoder Matrix
const double cm2count = 17.6839;
const double count2cm = 1/17.6839;

// left encoder variables
volatile unsigned char left_old, left_new; //Old0, New0;
volatile long left_position = 0; //Position0 = 0;


//right encoder variables
volatile unsigned char right_old, right_new; //Old0, New0;
volatile long right_position = 0; //Position0 = 0;
double right_position_cm=0;


NewPing sonar(TRIGGER_PIN, ECHO_PIN, 100);

double positionSP = 5;
long positionSPlong = positionSP*cm2count;
double positionPWM = 0;


//double Kp = 38;
//double Ki = 1;
//double Kd = 2;

//double Kp = 220.0;
//double Ki = 37;
//double Kd = 5;
//double Kp = 207; // 0.5, 90 deg phase
//double Ki = 16;

//double Kp = 79;
//double Ki = 77;
//double Kd = 2.4;

double Kp = 81; // 10 rad/sec, 75 phase margin
double Ki = 14;
double Kd = 3;

//double Kp = 38.5; // 5 rad/sec, 77 phase margin
//double Ki = 5;
//double Kd = 0;


int sampleTime = 20;
PID motorPID(&right_position_cm, &positionPWM, &positionSP,Kp,Ki,Kd, REVERSE);

unsigned long timeOld;
void setup()
{
  pinMode(LEFT_MOTOR_DIRECTION,OUTPUT);
  pinMode(LEFT_MOTOR_PWM,OUTPUT);
  pinMode(LEFT_ENCODER_CHA, INPUT);
  pinMode(LEFT_ENCODER_CHB, INPUT);

  
  pinMode(RIGHT_MOTOR_DIRECTION,OUTPUT);
  pinMode(RIGHT_MOTOR_PWM,OUTPUT);
  
  digitalWrite(LEFT_MOTOR_DIRECTION,LOW);
  analogWrite(LEFT_MOTOR_PWM,0);
  digitalWrite(RIGHT_MOTOR_DIRECTION,LOW);
  analogWrite(RIGHT_MOTOR_PWM,0);
  
  attachInterrupt(LEFT_ENCODER_INTERRUPT, left_encoder_interrupt, CHANGE); // interrupt on pin 2;
  attachInterrupt(RIGHT_ENCODER_INTERRUPT, right_encoder_interrupt, CHANGE); // interrupt on pin 2;
  
  motorPID.SetMode(AUTOMATIC);
  motorPID.SetSampleTime(sampleTime);
  motorPID.SetOutputLimits(-255,255);
  timeOld = millis();
  Serial.begin(9600);

}

/* Loop and print the time every second */
int dir = 0;

void loop(){
{

    motorPID.Compute();
    if (positionPWM > 0) {
        digitalWrite(LEFT_MOTOR_DIRECTION,HIGH);
        analogWrite(LEFT_MOTOR_PWM,positionPWM);
        digitalWrite(RIGHT_MOTOR_DIRECTION,HIGH);
        analogWrite(RIGHT_MOTOR_PWM,positionPWM);
    }
    else {
        digitalWrite(LEFT_MOTOR_DIRECTION,LOW);
        analogWrite(LEFT_MOTOR_PWM,-positionPWM);
        digitalWrite(RIGHT_MOTOR_DIRECTION,LOW);
        analogWrite(RIGHT_MOTOR_PWM,-positionPWM);
    }
  }
  
  unsigned long timeNow = millis();

  
  sprintf(buf, "%lu, %ld, %ld, %d \n", timeNow,  positionSPlong, right_position, (int)positionPWM);
//  sprintf(buf, "%lu, %lu \n", timeNow, right_position);

//  sprintf(buf, "%d, %d, \n", dir, right_position);
  Serial.print(buf);  

  if (timeNow - timeOld > 5000){
    positionSP = -positionSP;
    positionSPlong = -positionSPlong;
    timeOld = timeNow;
  }
//  delay(sampleTime);
}
      

// 

void left_encoder_interrupt(){
    left_old = left_new;
    left_new =  digitalRead (LEFT_ENCODER_CHA) * 2 + digitalRead (LEFT_ENCODER_CHB);           // Convert binary input to decimal value
    left_position += QEM [left_old * 4 + left_new];
}

void right_encoder_interrupt(){
    right_old = right_new;
    right_new =  digitalRead (RIGHT_ENCODER_CHA) * 2 + digitalRead (RIGHT_ENCODER_CHB);           // Convert binary input to decimal value
    right_position += QEM [right_old * 4 + right_new];
    right_position_cm = right_position*count2cm;
}



